using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

namespace HL7Viewer
{
    public class DataProcessing
    {
        private MainWindow displayForm;

        public DataProcessing( MainWindow displayForm )
        {
            this.displayForm = displayForm;
        }

        public void DisplayDataset( HL7DataSet hl7DataSet )
        {
            DataSet myDataSet = new DataSet( "hl7DataSet" );
            DataTable tDataSetTable = new DataTable( "HL7 Datensatz" );
            DataColumn cFieldValue = new DataColumn( "Feldinhalt", typeof( string ) );
            DataColumn cFieldType = new DataColumn( "Feldtype", typeof( string ));
            DataColumn cFieldNumber = new DataColumn( "Feldnummer", typeof( string ));
            DataGridTableStyle gridStyle = new DataGridTableStyle();
            DataRow row1;

            // Einen neuen TableStyle erzeugen und der Tabelle zuweisen
            gridStyle.MappingName = "HL7 Datensatz";
            gridStyle.AllowSorting = false;
            gridStyle.RowHeadersVisible = false;
            displayForm.dataGrid1.TableStyles.Clear();
            displayForm.dataGrid1.TableStyles.Add( gridStyle );

            // Drei neue Spalten zur Tabelle hinzufgen
            tDataSetTable.Columns.Add( cFieldType );
            tDataSetTable.Columns.Add( cFieldNumber );
            tDataSetTable.Columns.Add( cFieldValue );

            // Die Zeilen zur Tabelle hinzufgen
            string fieldPrefix = "MSH";
            int fieldCount = 1;
            for( int i = 0; i < hl7DataSet.dataFields.Length; i ++ )
            {                
                if( hl7DataSet.dataFields[i] != "" )
                {                  
                    row1 = tDataSetTable.NewRow();

                    if( hl7DataSet.dataFields[i].Contains( "[PREFIX]" ))
                    {
                        fieldPrefix = hl7DataSet.dataFields[i].Replace( "[PREFIX]", "" );
                        fieldCount = 0;
                    }
                    
                    if( fieldCount == 0 )
                    {
                        row1["Feldnummer"] = "";
                        row1["Feldtype"] = "";
                    }
                    else
                    {
                        row1["Feldnummer"] = fieldCount;
                        row1["Feldtype"] = fieldPrefix;
                    }
                    row1["Feldinhalt"] = hl7DataSet.dataFields[i].Replace( "[PREFIX]", "" );

                    tDataSetTable.Rows.Add( row1 );
                }
                fieldCount ++;
            }

            // Die Tabelle zum Dataset hinzufgen
            myDataSet.Tables.Add( tDataSetTable );

            // Maximale Breite der Spalten ermitteln
            int lenContentField = LongestField( myDataSet, "HL7 Datensatz", "Feldinhalt" );
            int lenTypeField = LongestField( myDataSet, "HL7 Datensatz", "Feldtype" );
            int lenNumberField = LongestField( myDataSet, "HL7 Datensatz", "Feldnummer" );

            // Datenbindung zwischen dem Grid und dem Dataset erstellen
            displayForm.dataGrid1.SetDataBinding( myDataSet, "HL7 Datensatz" );

            // Spaltenbreite setzen
            displayForm.dataGrid1.TableStyles["HL7 Datensatz"].GridColumnStyles["Feldtype"].Width = lenTypeField;
            displayForm.dataGrid1.TableStyles["HL7 Datensatz"].GridColumnStyles["Feldinhalt"].Width = lenContentField;
            displayForm.dataGrid1.TableStyles["HL7 Datensatz"].GridColumnStyles["Feldnummer"].Width = lenNumberField;
        }

        private int LongestField( DataSet ds, string TableName, string ColumnName )
        {
            int maxlength = 0;
            int tot = ds.Tables[TableName].Rows.Count;
            string straux = "";
            int intaux = 0;

            Graphics g = displayForm.dataGrid1.CreateGraphics();

            // Take width one balnk space to add to the new width to the Column
            int offset = Convert.ToInt32( Math.Ceiling( g.MeasureString( " ", displayForm.dataGrid1.Font ).Width ) );

            for( int i = 0; i < tot; ++i )
            {
                straux = ds.Tables[TableName].Rows[i][ColumnName].ToString();

                // Get the width of Current Field String according to the Font
                intaux = Convert.ToInt32( Math.Ceiling( g.MeasureString( straux, displayForm.dataGrid1.Font ).Width ) );
                if( intaux > maxlength )
                {
                    maxlength = intaux;
                }
            }// End of For Loop

            return maxlength + offset;
        }
    }
}
