using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace HL7Viewer
{
    public partial class MainWindow : Form
    {
        HL7FileReader hl7DataFile;
        HL7DataSet hl7DataSet;

        public MainWindow()
        {
            InitializeComponent();

            button1.Enabled = false;
            button2.Enabled = false;
            button3.Enabled = false;
            button4.Enabled = false;
            toolStripButton2.Enabled = false;
            label1.Visible = false;
            label3.Text = "Keine Daten geladen";
            hl7DataFile = null;
            hl7DataSet = null;
        }

        private void beendenToolStripMenuItem_Click( object sender, EventArgs e )
        {
            Environment.Exit( 0 );
        }

        private void ffnenToolStripMenuItem_Click( object sender, EventArgs e )
        {
            OpenFileDialog openFile = new OpenFileDialog();

            openFile.InitialDirectory = "c:\\";
            openFile.Filter = "HL7 Data Files (*.*)|*.*";
            openFile.FilterIndex = 2;
            openFile.RestoreDirectory = true;

            if( openFile.ShowDialog() == DialogResult.OK )
            {
                OpenFile( openFile.FileName );
            }
        }

        // Button: Zum nchsten Datensatz springen
        private void button2_Click( object sender, EventArgs e )
        {
            // berprfung ob wir schon beim letzten Datensatz angekommen sind
            if(( hl7DataFile.GetCurDatasetIndex + 1 ) < hl7DataFile.GetNumDatasets )
            {
                HL7DataSet hl7DataSet = hl7DataFile.GetNextDataset;
                if( hl7DataSet != null )
                {
                    DataProcessing dp = new DataProcessing( this );
                    dp.DisplayDataset( hl7DataSet );
                }
                label3.Text = ( hl7DataFile.GetCurDatasetIndex + 1 ).ToString() + "/" + hl7DataFile.GetNumDatasets.ToString();
            }
            // Buttons entsprechend aktivieren/deaktivieren    
            SetNavButtonState();
        }

        // Button: Zum vorherigen Datensatz springen
        private void button1_Click( object sender, EventArgs e )
        {
            HL7DataSet hl7DataSet = hl7DataFile.GetPrevDataset;
            if( hl7DataSet != null )
            {
                DataProcessing dp = new DataProcessing( this );
                dp.DisplayDataset( hl7DataSet );
            }
            label3.Text = ( hl7DataFile.GetCurDatasetIndex + 1 ).ToString() + "/" + hl7DataFile.GetNumDatasets.ToString();
            
            // Buttons entsprechend aktivieren/deaktivieren
            SetNavButtonState();
        }

        // Button: Zum ersten Datensatz springen
        private void button3_Click( object sender, EventArgs e )
        {
            HL7DataSet hl7DataSet = hl7DataFile.GetFirstDataset;
            if( hl7DataSet != null )
            {
                DataProcessing dp = new DataProcessing( this );
                dp.DisplayDataset( hl7DataSet );
            }
            label3.Text = ( hl7DataFile.GetCurDatasetIndex + 1 ).ToString() + "/" + hl7DataFile.GetNumDatasets.ToString();
            SetNavButtonState();
        }

        // Button: Zum letzten Datensatz springen
        private void button4_Click( object sender, EventArgs e )
        {
            HL7DataSet hl7DataSet = hl7DataFile.GetLastDataset;
            if( hl7DataSet != null )
            {
                DataProcessing dp = new DataProcessing( this );
                dp.DisplayDataset( hl7DataSet );
            }
            label3.Text = ( hl7DataFile.GetCurDatasetIndex + 1 ).ToString() + "/" + hl7DataFile.GetNumDatasets.ToString();
            SetNavButtonState();
        }

        // Men: ? -> ber
        private void berToolStripMenuItem_Click( object sender, EventArgs e )
        {
            AboutBox about = new AboutBox();

            about.ShowDialog();
        }

        // Menleiste Datei ffnen
        private void toolStripButton1_Click( object sender, EventArgs e )
        {
            ffnenToolStripMenuItem_Click( sender, e );
        }

        private void MainWindow_Resize( object sender, EventArgs e )
        {
            dataGrid1.Size = new System.Drawing.Size( this.Size.Width - 197, this.Size.Height - 113 );
            button1.Location = new Point( this.Size.Width - 179, button1.Location.Y );
            button2.Location = new Point( this.Size.Width - 179, button2.Location.Y );
            button3.Location = new Point( this.Size.Width - 179, button3.Location.Y );
            button4.Location = new Point( this.Size.Width - 179, button4.Location.Y );
            label2.Location = new Point( this.Size.Width - 179, label2.Location.Y );
            label1.Location = new Point( this.Size.Width - 179, label1.Location.Y );
            label3.Location = new Point( this.Size.Width - 119, label3.Location.Y );
        }

        // Toolbar Button: Filter setzen
        private void toolStripButton2_Click( object sender, EventArgs e )
        {
            Filter filterDialog = new Filter( hl7DataFile );
            if( filterDialog.ShowDialog() != DialogResult.Abort )
            {
                HL7DataSet hl7DataSet = hl7DataFile.GetFirstDataset;
                if( hl7DataSet != null )
                {
                    DataProcessing dp = new DataProcessing( this );
                    dp.DisplayDataset( hl7DataSet );
                }
                label3.Text = ( hl7DataFile.GetCurDatasetIndex + 1 ).ToString() + "/" + hl7DataFile.GetNumDatasets.ToString();

                if( hl7DataFile.Filter != null && hl7DataFile.Filter != "" )
                {
                    label1.Visible = true;
                }
                else
                {
                    label1.Visible = false;
                }
            }
        }

        // Buttons je nach Datensatz setzen
        private void SetNavButtonState()
        {
            if(( hl7DataFile.GetCurDatasetIndex + 1 ) == 1 )
            {
                button1.Enabled = false;
                if(( hl7DataFile.GetCurDatasetIndex + 1 ) != hl7DataFile.GetNumDatasets )
                {
                    button2.Enabled = true;
                }
            }
            else
            {
                button1.Enabled = true;
            }

            if(( hl7DataFile.GetCurDatasetIndex + 1 ) == hl7DataFile.GetNumDatasets )
            {
                button2.Enabled = false;                
            }
            else
            {
                button2.Enabled = true;
            }

            button3.Enabled = true;
            button4.Enabled = true;
        }

        // Drag und Drop Funktionalitt
        private void MainWindow_DragOver( object sender, DragEventArgs e )
        {
            e.Effect = DragDropEffects.Copy;
        }

        private void MainWindow_DragDrop( object sender, DragEventArgs e )
        {
            if( e.Data.GetDataPresent( DataFormats.FileDrop, true ))
            {
                string fileName;

                fileName = (( string[] ) e.Data.GetData( DataFormats.FileDrop ))[0];
                OpenFile( fileName );
            }
        }

        // ffnen einer Datei
        private bool OpenFile( string filePath )
        {
            try
            {
                hl7DataFile = new HL7FileReader( filePath );
            }
            catch( Exception Hl7Exception )
            {
                MessageBox.Show( "Keine HL7 Datendatei", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error );
                return false;
            }

            toolStripStatusLabel1.Text = filePath + ": " + hl7DataFile.GetNumDatasets.ToString() + " Datenstze";

            if( hl7DataFile.GetNumDatasets > 0 )
            {
                hl7DataSet = hl7DataFile.GetFirstDataset;
                DataProcessing dp = new DataProcessing( this );
                dp.DisplayDataset( hl7DataSet );

                string[] fileName = filePath.Split( "\\".ToCharArray() );
                string captionName = fileName[fileName.Length - 1];

                dataGrid1.CaptionText = captionName;

                label3.Text = ( hl7DataFile.GetCurDatasetIndex + 1 ).ToString() + "/" + hl7DataFile.GetNumDatasets.ToString();

                toolStripButton2.Enabled = true;
                SetNavButtonState();
            }

            return true;
        }
    }
}